//
// Copyright (c) 1999 Microsoft Corp.  All Rights Reserved


#include <fstream.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

#ifdef WIN32
#include <io.h>
#include <strstrea.h>
#define IOS_BINARY ios::binary
#endif

#ifdef UNIX
#include <strstream.h>
#define IOS_BINARY ios::bin
#endif

#include "votefile.h"
#include "votecount.h"

const int QueryNumDecodes = 2;
const char QueryFromDecode[QueryNumDecodes] = {'+', '/'};
const char QueryToDecode[QueryNumDecodes] = {' ', DIR_SEPARATOR};

///////////////////////////////////////////////////////////////////
char *QueryDecode(char *queryString)
{
	// Performs in-place decoding of the query string.
	// Additional chars to translate/decode should be appended to the
	// QueryFrom/ToDecode arrays
    
	// Simple char by char decoding.
    if (queryString == NULL)
        return NULL;
    for (char *p = queryString; *p; p++)
    {    
        for (int i=0; i<QueryNumDecodes; i++)
        {
            if (*p == QueryFromDecode[i])
            {
                *p = QueryToDecode[i];
                break;
            }
        }
    }
    return queryString;
}

///////////////////////////////////////////////////////////////////
char* ParseRequest(const int Length, BOOL error = FALSE)
{    
	// This function reads the request string received from the client
	// in stdin into RequestBuffer, performing necessary hex conversions
	// and platform-specific char translations along the way.
	// Callers should delete the pointer themselves.

	char *pRequestBuffer = new char [Length + 1]; // +1 for '\0'
	if (!pRequestBuffer) return NULL;

	char *pResult = pRequestBuffer;
    int intcast = 0;
	char ch;

	for(int i = 0; (i < Length) && ((ch = getchar()) != EOF) ; i++)
	{	
		if ((ch == '%') && (i + 1 < Length))
		{	
			char encoded[3];
            encoded[0] = getchar();
            encoded[1] = getchar();
            encoded[2] = 0;
            sscanf(encoded, "%x", &intcast);
			*pRequestBuffer = (char)intcast;
			i +=2;  // increment lcv to reflect extra getchars
		}   
		else *pRequestBuffer = (char)ch;
		pRequestBuffer++;
	}
	*pRequestBuffer = '\0';

	clearerr(stdin); // if our parsing caused anomalies in stdin, clear
	return pResult;
}

///////////////////////////////////////////////////////////////////
void GetKeys(char* pRequestBuffer, char** VoteResult, char** VoteResultsFileName)
{
	// This function parses the RequestBuffer and returns the values
	// corresponding to the keys "Vote" and "Path".
	// warning: this function returns pointers into RequestBuffer for perf
	//          reasons, but may destroy "rest of RequestBuffer" in
	//          the process
	
	char *pKeyBegin = pRequestBuffer;
    char *pKeyEnd = NULL;    
	char *Value = NULL;
	
	for (char *p = pRequestBuffer; *p;)
    {
        if (*p == '=')        
            pKeyEnd = p;
        // If it's a key divider, or the last char of the string,
        // parse the entire key/value pair.  But if we're at the end
        // of the string, move p there so that the rest of the code
        // works as expected.
        if (*(p+1) == '\0')
            p++;
        if (*p == '&' || *p == '\0')
        {            
            // At this point, key is from pKeyBegin to pKeyEnd-1,
            // and value is from pKeyEnd+1 to p-1.
            if (pKeyEnd)
            {
                int keyLen = pKeyEnd - pKeyBegin;    
                if (keyLen >0)
                {
                    // Replace the & seperator with a '\0' so matching
					// value can be returned w/o requiring a strcpy
					*p = '\0';
                    if (strncmp("Path", pKeyBegin, keyLen) == 0)            
                        *VoteResultsFileName = pKeyEnd+1;            
                    else if(strncmp("Vote", pKeyBegin, keyLen) == 0)                
                        *VoteResult = pKeyEnd+1;               
                }
            }
            // Reset the pointers:
            pKeyBegin = p+1;
            pKeyEnd = NULL;
        }
        p++;    
	}    
}

///////////////////////////////////////////////////////////////////
int main(int /* argc */, char ** /* argv */)
{
	// comment out (unused) argc and argv to prevent compiler warnings.
    
	div_t percentage;        // to tally voting percentages
	
	const char* path = getenv("PATH_TRANSLATED");
    const char* RequestLength = getenv("CONTENT_LENGTH");
	
	char* pInputBuffer = ParseRequest(atoi(RequestLength));    
	
    char* VoteResultsFileName = "";    
    char* VoteResult = "";
    
	// Parse the query string for "Vote" and "Path" values
	GetKeys(QueryDecode(pInputBuffer), &VoteResult, &VoteResultsFileName);

	// If we couldn't find the 'Path' key in request buffer, quit
    if (VoteResultsFileName == NULL || strlen(VoteResultsFileName) == 0)
    {     return 1;    }
    
    // The constructor waits until the file is no longer locked, then locks it.
    CVote *VoteCntFile = new CVote(VoteResultsFileName);        
    
	// percentage.quot * 100 == final percentage.
	if (!strcmp(VoteResult,"V1"))
	{	VoteCntFile->Increment("vote1type");	
		percentage = div ((VoteCntFile->GetVoteCount("vote1type")* 100),
						(VoteCntFile->GetVoteCount("vote1type") +
						VoteCntFile->GetVoteCount("vote2type")));
						};    
    	
	if (!strcmp(VoteResult,"V2"))
	{	VoteCntFile->Increment("vote2type");	    
    	percentage = div ((VoteCntFile->GetVoteCount("vote2type")* 100),
						(VoteCntFile->GetVoteCount("vote1type") +
						VoteCntFile->GetVoteCount("vote2type")));
						};    
	
	// Destruct as soon as possible to unlock the file so others can access it.
    delete VoteCntFile;

	// return vote results for display in browser
#ifdef WIN32
    _setmode( _fileno( stdout ), _O_BINARY );    // Make stdout binary
#endif
#ifdef UNIX
    fdopen( fileno(stdout), "wb");               // Make stdout binary
#endif

	// output MIME header so browsers know that we're HTML.
	fprintf(stdout, "Content-type: text/html\n\n");
	
	fflush(stdout);

	cout<< "<html>\n"
		<< "<head>\n"
		<< "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=None\">"
		<< "\n<title>Vote Confirmation</title>\n</head>\n<body>\n" 
        << "<h1 align=\"center\"><strong>Vote Confirmation "
		<< "</strong></h1>\n\n<p align=\"center\">&nbsp;</p>"
		<< "<p align=\"center\"><font size=\"3\">\n"
		<< "Your ballot has been cast and your vote "
		<< "has been registered.</p>\n"
		<< "<p align=\"center\">Currently, " << (percentage.quot) 
		<< "% of the voting population agrees with your vote.</font></p>"
		<< "\n</body>\n\n</html>";
		
	fflush(stdout);
	
	//delete VoteResultsFileName; 
	delete pInputBuffer;
    return 0;
}