//
// Copyright (c) 1999 Microsoft Corp.  All Rights Reserved

// votecount.cpp : Defines the initialization routines for the DLL.
//

#include "votecount.h"
#include "votefile.h"
#include "../../webbot.h"
#include <string.h>
#include <stdlib.h>

#ifdef UNIX
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#else
#include <direct.h>
#include <io.h>
#endif


//////////////////////////////////////////////////////////////////
// Utility URL and filename manipulation functions.

//////////////////////////////////////////////////////////////////
void CopyPageDirName(const char *PageURL, char *dest, char translateChar)

// This function copies the PageURL directory name (the PageURL
// minus the page itself) into the destination.  It also translates
// the '/' character into the character specified by translateChar.  
// This is useful if the desired final form is to contain '\' or ':'
// as the directory separator.  
// If the passed PageURL is NULL, it will remain unmodified.

// This function returns the number of characters copied, not
// counting the '\0' at the end 

// NOTE: this routine assumes enough space has been allocated in the 
// destination buffer to hold the *entire* PageURL, not just enough up
// to the '/' of the PageURL.
//
// Example: PageURL="Subweb/simple/index.htm" and translateChar is '\' then
// the result in dest is "Subweb\simple"

{
    if (!PageURL)
        return;
    const char *pSource = PageURL;    
    char *pDest = dest;
    char *last = pDest;
    for (; *pSource; pSource++, pDest++)
    {
        if (*pSource == '/')
        {
            last = pDest;
            *pDest = translateChar;
        }
        else
            *pDest = *pSource;
    }

    // last points to the last '/' in the PageURL, or the beginning of it if
    // there are no '/'s in the string.  Thus, close off the string at
    // this location.

    // if characters were copied then append a dir seperator to the end
    // of the whole thing.  Otherwise, just close off the string    
    if (last != dest)
    {
        *last = translateChar;
        *(last+1) = '\0';
    }
    else    
        *last = '\0';            
}

///////////////////////////////////////////////////////////////////
const char *SubWebName(CWebBotDict &bot)
{
    // This function returns the subweb name, taken from the WebURL.
    // The function assumes WebURL is in the form:
    // protocol://host/subweb
    //
    // Method to find the subweb: Scan for the first / which
    // does not have a / following it.  
    const char *WebURL = bot.GetValue("WebURL");
    if (!WebURL)
        return "";
    
    const char *p;
    for (p = WebURL; *p; p++)
    {
        if (*p=='/')
        {
            if (*(p+1) != '/')
                return p;
            else
                p++;            // Skip over next spot
        }
    }
    return p;

}


///////////////////////////////////////////////////////////////////
char *TranslateChar(const char *PageURL, char translateFROMChar, char translateTOChar)
{
	// returns a string that is PageURL w/ every occurrence of translateFROMChar replaced
	// with translateTOChar
	// users of this function should delete the pointer after use

    if (!PageURL)
        return "";
    
	char *pTranslatedString = new char[strlen(PageURL) + 1]; // + 1 for \0
	char *pTransStringloc = pTranslatedString;
	
    for (; *PageURL; PageURL++, pTransStringloc++)
    {
        if (*PageURL == translateFROMChar) {*pTransStringloc = translateTOChar; }
		else { *pTransStringloc = *PageURL; }
	}

	*pTransStringloc = '\0';

	return pTranslatedString;
}

///////////////////////////////////////////////////////////////////
const char *ReturnBaseFolder(const char* sFilePath)
{
    // returns a URL that equals sPageURL with everything after the trailing / snipped.
  	// users of this function should delete the pointer after use
	
	if (!sFilePath)
        return "";
    
	const char* last = strrchr(sFilePath,'\\'); 

	if (last == NULL)
		return "";

	char* newstring = new char[(last-sFilePath)+1];
	strncpy (newstring, sFilePath, last-sFilePath); 	
	newstring[(last-sFilePath)]= '\0';  // close off newstring
	return newstring;
}

///////////////////////////////////////////////////////////////////
const char *mkdirNested(const char* sPageURL)
{
	// recursively creates the directory structure specified by sPageURL

	if (access(ReturnBaseFolder(sPageURL), 0) != 0) { return sPageURL; }
	else 
	{	mkdirNested(ReturnBaseFolder(sPageURL)); 
#ifdef UNIX
        mkdir(sPageURL, 0777);
#else
        mkdir(sPageURL);
#endif
		return sPageURL;
	}
}

///////////////////////////////////////////////////////////////////
// Vote count file functions

void CreateAndResetVoteResultsFile(const char *VoteCntFile, long resetValue=0)
{
    // Create a CVote object for dealing with the vote results file.
    // Waits until the file is available, then locks it.  Times out
    // if the file does not become available in a certain amount of time.
    // If the file does not exist, it will create it.
   
	// If new vote types are added by end-users,
	// and they have different default values, the second parameter will
	// come in handy.  For now, just stifle compiler warnings.

	(void)resetValue;

	CVote votefile(VoteCntFile, TRUE);
    votefile.SetVoteCount("vote1type", 0);
	votefile.SetVoteCount("vote2type", 0);

    // CVote destructor writes and unlocks the file
}

///////////////////////////////////////////////////////////////////
void HandleVoteCntFile(CWebBotDict &bot)
{    
    // Figures out the filename for the vote results file.  
    // and then creates it and the directory structure under _private
	// if it doesn't exist.

   	// VoteResultsPath = DocumentRoot + \_private\ + PageURL + .vot

    const char *PageURL = bot.GetValue("PageURL");
    const char *DocumentRoot = bot.GetValue("DocumentRoot");    
	char *TranslatedString = TranslateChar(PageURL,'/','\\');

    if (!PageURL || !DocumentRoot)
        return;

    // Create a buffer large enough:
    int DocumentRootLen = strlen(DocumentRoot);
    char *PageDirName = new char[DocumentRootLen  
                            + 10    +                // +10 for "\_private\"
                            strlen(PageURL)          // +1 for DIR_SEPARATOR
                            + 5];    // +5 for ".vot"+'\0'
        
	// Construct the path of the .vote file:
    strcpy(PageDirName, DocumentRoot);
	strcat(PageDirName, "\\_private\\");
	strcat(PageDirName, TranslatedString);
	strcat(PageDirName, ".vot");
	
    ////////////////////////////////////////////////////
    // Now buffer holds the filename of the vote count file.
	bot.SetValue("U-votepath",PageDirName);
    
	//Check for existence of, and create directory structure if necessary
    if (access(PageDirName, 0) != 0) 
    {	mkdirNested(ReturnBaseFolder(PageDirName));    }
    
	// Check for existence of, and create file if necessary
    if (access(PageDirName, 0) != 0)
	{	CreateAndResetVoteResultsFile(PageDirName);	}

    delete[] PageDirName;
	delete TranslatedString;
}

///////////////////////////////////////////////////////////////////
const int QueryNumTranslations = 2;
const char QueryTranslateFrom[QueryNumTranslations] = {' ', '\\'};
const char *QueryTranslateTo[QueryNumTranslations] = {"+", "/"};

const int URLNumTranslations = 2;
const char URLTranslateFrom[URLNumTranslations] = {' ', '\\'};
const char *URLTranslateTo[URLNumTranslations] = {"%20", "/"};

///////////////////////////////////////////////////////////////////
char *URLEncode(const char *url, 
                const char *translateFrom, 
                const char **translateTo, 
                int numTranslations)
{
    // Note: Users of this function should delete the pointer themselves
    if (url == NULL)
        return NULL;

    // maximum size of the encoded string is the url's len*3.  This assumes that
    // no TranslateTo string is more than 3 chars, which will always be
    // true for URL encoding
    char *encoded = new char[strlen(url)*3 + 1];            // +1 for '\0'

    char *currentTo = encoded;
    BOOL found;
    for (const char *currentFrom = url; *currentFrom; currentFrom++)
    {
        found = FALSE;
        for (int i = 0; i < numTranslations && !found; i++)
        {
            if (*currentFrom == translateFrom[i])
            {
                // Matches a char, copy over the translateTo for this char
                strcpy(currentTo, translateTo[i]);
                currentTo+=strlen(translateTo[i]);
                found = TRUE;
            }
        }

        if (!found)
        {
            // Wasn't a char to translate, just do plain vanilla copy
            *currentTo = *currentFrom;
            currentTo++;
        }
    }
    // Close it off
    *currentTo = '\0';    
    return encoded;
}


//////////////////////////////////////////////////////////////////
// DLL Entry point for expanding the Custom Component (bot)

//////////////////////////////////////////////////////////////////
BeginWebBotExpand(votebot,ret,bot,cgi,form)
{
    // Squelch compiler warnings
    (void)cgi;
    (void)form;

    // Take care of the vote results file
    HandleVoteCntFile(bot);

    // Create the HTML tag which will call the cgi program "votebot"

    // First, generate the PageDirName
    const char *PageURL = bot.GetValue("PageURL");
    if (!PageURL)
        PageURL = "";

    // The +2 is for the potentially extra '/' at the end, and for the '\0'
    char *PageDirName = new char[strlen(PageURL) + 2];
    CopyPageDirName(PageURL, PageDirName, '/');

    // Create the URL in ret, URL encode it into "url".
    ret.Clear();
    ret.Append("/cgi-bin/votebot.exe");
    ret.Append(SubWebName(bot));
    ret.Append("/");    
    ret.Append(PageDirName);
    
	char *url = URLEncode(ret.GetContents(), 
                            URLTranslateFrom, 
                            URLTranslateTo, 
                            URLNumTranslations);        

	// Put in the MIME Links: header so that FP Explorer knows
    // that the Bot is linking to the vote results file.  That is, append
    // Links: WebURL/_private/PageURL.vot, URL encoding the filename
    ret.Clear();
    ret.Append("Links: ");
    ret.Append(bot.GetValue("WebURL"));
	ret.Append("/_private/");
	char *EncodedPageFileName = URLEncode(PageURL,
                                          URLTranslateFrom,
                                          URLTranslateTo,
                                          URLNumTranslations);
    ret.Append(EncodedPageFileName);
    ret.Append(".vot");

    // Put in the MIME WriteLinks: header so that the explorer knows
    // that our Bot is linking to the vote results file.  This will cause  
	// the server extensions to set the permissions on this
	// file so that browsers can write to it.  
    // WriteLinks: WebURL/_private/PageURL.vot.  
	ret.Append("\nWriteLinks: ");
	ret.Append(bot.GetValue("WebURL"));
	ret.Append("/_private/");
	ret.Append(EncodedPageFileName);
	ret.Append(".vot");	

    // Put in the MIME Error: header to detail any setup errors
	const char *sztype = bot.GetValue("S-VOTETYPE");
	const char *szopt  = bot.GetValue("S-VOTEOPT");
    if (!sztype || !szopt)
        ret.Append("\nError: A vote component is incorrectly configured.  "
                   "You must provide values for the \"S-VOTETYPE\" and "
                   "\"S-VOTEOPT\" parameter(s).");

	// a blank like signifies the end of all MIME headers:
#ifdef UNIX
	ret.Append("\r\n\r\n");
#else
	ret.Append("\n\n");
#endif

	// place the vote submit form onto the page.
	ret.Append("<table border=\"1\" width=\"100%\" bordercolordark=\"#000000\"");
	ret.Append("bgcolor=\"#FFFFFF\">\n");
	ret.Append("<tr>\n");
    ret.Append("<td width=\"100%\">\n");
	
	// for CGI implementation:
	ret.Append("<form method=\"POST\" action=\"");
	
	// Append the name of the CGI executable (vote submission handler)
	ret.Append(url);

	ret.Append("\" target=\"_top\">\n");
    ret.Append("\n<input type=\"hidden\" name=\"Path\" value=\"");
    ret.Append(bot.GetValue("U-votepath"));
    
    ret.Append("\">\n<p align=\"center\">");
    ret.Append("How do you feel about the ");	
		
	if (sztype && !strcmp(sztype,"OPINION"))
        ret.Append("opinion");
	
	else if (sztype && !strcmp(sztype,"PROPOSAL"))
		ret.Append("proposal");
		
	ret.Append(" expressed on this page?</p>");
    ret.Append("\n<p align=\"center\"><input");
    ret.Append("\ntype=\"radio\" value=\"V1\" checked name=\"Vote\">");
	
	if (szopt && !strcmp(szopt,"AGREE"))
	{
        ret.Append(" I agree! &nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <input");
		ret.Append("\ntype=\"radio\" name=\"Vote\" value=\"V2\"> I disagree!");
    }
	
	else if (szopt && !strcmp(szopt,"YES/NO"))
	{
        ret.Append(" Yes! &nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <input");
		ret.Append("\ntype=\"radio\" name=\"Vote\" value=\"V2\"> No!");
    }
	
	else if (szopt && !strcmp(szopt,"FOR/AGAINST"))
	{
        ret.Append(" For! &nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <input");
		ret.Append("\ntype=\"radio\" name=\"Vote\" value=\"V2\"> Against!");
    };
		
	ret.Append("\n</p>");	
    ret.Append("\n<p align=\"center\"><input type=\"submit\"");
    ret.Append("value=\"Cast Your Ballot!\" name=\"B1\"></p>");
    ret.Append("\n</form>");

	// Close off the table
	ret.Append("</td>\n</tr>\n</table>\n");

	// Cleanup
    delete[] PageDirName;
    delete[] EncodedPageFileName;
    delete[] url;
}
EndWebBotExpand



