
// fproot - sample program to determine what version of FrontPage
// is installed and where custom components should be installed

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <mbstring.h>
#include <msi.h>
#include <shlobj.h>

// friendly names for different versions of FrontPage
#define FP_2K   "FrontPage 2000"
#define FP_98   "FrontPage 98"
#define FP_97   "FrontPage 97"
#define FP_11   "FrontPage 1.1"
#define FP_10   "FrontPage 1.0"

// MSI GUID for FrontPage
#define FP_2000_GUID "{3479E821-B61B-11D1-998C-00A0C90A43B3}"

//----------------------------------------------------------------------------
// getComponentPath()
// 
//   utility function - calls MSI to find out a component path.
// 
//   szComponentGUID (input) points to an MSI component GUID string.
//   szComponentPath (output) points to a buffer at least _MAX_PATH chars long.
//----------------------------------------------------------------------------

static BOOL getComponentPath(char *szComponentGUID,
                             char *szComponentPath)
{
    BOOL fRet = FALSE;
    *szComponentPath = 0;
    HMODULE hmodMSI = LoadLibrary("msi.dll");
    if (hmodMSI)
    {
        UINT (WINAPI *pMsiGetProductCode)(LPCSTR szComponent,
                                          LPSTR lpProductBuf);
        INSTALLSTATE (WINAPI *pMsiGetComponentPath)(LPCSTR szProduct,
                                                    LPCSTR szComponent,
                                                    LPSTR lpPathBuf,
                                                    DWORD *pcchBuf);
        (FARPROC&) pMsiGetProductCode =
            GetProcAddress(hmodMSI, "MsiGetProductCodeA");
        (FARPROC&) pMsiGetComponentPath =
            GetProcAddress(hmodMSI, "MsiGetComponentPathA");

        fRet = (pMsiGetProductCode && pMsiGetComponentPath);

        // This is a guid, so FP_2000_GUID is just a placeholder
        char szProductCode[ sizeof(FP_2000_GUID) ];
        DWORD cb = _MAX_PATH;
        if (fRet)
            fRet = (pMsiGetProductCode(szComponentGUID, szProductCode)
                    == ERROR_SUCCESS);
        if (fRet)
        {
            INSTALLSTATE s;
            s = pMsiGetComponentPath(szProductCode, szComponentGUID,
                                     szComponentPath, &cb);
            fRet = (s == INSTALLSTATE_LOCAL) || (s == INSTALLSTATE_SOURCE);
        }

        if (fRet)
        {
            // we now have key file.  Strip key file.
            unsigned char *szSlash = _mbsrchr((unsigned char*) szComponentPath,
                                              '\\');
            if (szSlash)
                *szSlash = 0;
        }
        FreeLibrary(hmodMSI);
    }

	return fRet;
}

//----------------------------------------------------------------------------
// getOldFrontPageRoot()
//
//   utility function - determine where FrontPage is installed 
//   for versions prior to FP2K. 
//
//   szPath (output) points to a buffer at least _MAX_PATH chars long.
//   szVersion (output) points to a buffer at least 32 chars long.
//----------------------------------------------------------------------------

static BOOL getOldFrontPageRoot(char *szPath, char *szVersion)
{
	// look for old roots in "frontpg.ini" or "vermeer.ini"

	static struct _fpversion
	{
		char *sectionName;
		char *friendlyName;
	} versions[] =
	{
		{ "FrontPage 3.0", FP_98 },
		{ "FrontPage 2.0", FP_97 },
		{ "FrontPage 1.1", FP_11 },
	};
	int nVersions = sizeof(versions) / sizeof(struct _fpversion);

	DWORD dwCount = 0;

	// check for previous versions of FrontPage in descending order

	for(int i=0; dwCount == 0 && i < nVersions; i++)
	{
		dwCount = GetPrivateProfileString(versions[i].sectionName,  // address of section name 
    									  "FrontPageRoot", // address of key name 
    									  NULL,	           // address of default string 
    									  szPath,		   // address of destination buffer 
    									  _MAX_PATH-1,	   // size of destination buffer 
    									  "frontpg.ini");  // address of initialization filename 
		if(dwCount > 0)
			strcpy(szVersion, versions[i].friendlyName);
	}

	// make one last check for FrontPage 1.0
	if(dwCount == 0)
	{
		dwCount = GetPrivateProfileString("FrontPage",     // address of section name 
    									  "FrontPageRoot", // address of key name 
    									  NULL,	           // address of default string 
    									  szPath,		   // address of destination buffer 
    									  _MAX_PATH-1,	   // size of destination buffer 
    									  "vermeer.ini");  // address of initialization filename 
		if(dwCount > 0)
			strcpy(szVersion, FP_10);
	}

	return (dwCount > 0);
}

//----------------------------------------------------------------------------
// GetFrontPageRoot()
//
//   determine where latest version of FrontPage is installed; 
//   return the install path and readable version string.
//
//   szPath (output) points to a buffer at least _MAX_PATH chars long.
//   szVersion (output) points to a buffer at least 32 chars long.
//----------------------------------------------------------------------------

BOOL GetFrontPageRoot(char *szPath, char *szVersion)
{
	// first try FrontPage 2000 method using MSI (Windows installer)
    if(getComponentPath(FP_2000_GUID, szPath))
	{
		strcpy(szVersion,FP_2K);
		return TRUE;
	}

	// try old method using Windows INI file
	return getOldFrontPageRoot(szPath, szVersion);
}

int main()
{
    static char szRootPath[_MAX_PATH];
	static char szVersion[128];

	static char *pszBots = "bots=%s\\bots\n";
	static char *pszPages = "pages=%s\\pages\n";
	static char *pszWebs = "webs=%s\\webs\n";
	static char *pszFrames = "frames=%s\\frames\n";
	static char *pszCSS = "css=%s\\css\n";
	static char *pszThemes = "themes=%s\\themes\n";
 
	if(GetFrontPageRoot(szRootPath, szVersion))
	{
		printf("Version=%s\n",szVersion);
		printf("FrontPageRoot=%s\n\n",szRootPath);

		printf("---------------------------------------\n");
		printf("Installation Dirs for Custom Components\n");
		printf("---------------------------------------\n");

		if(!strcmp(szVersion,FP_2K))
		{
			HRESULT ret;
			LPITEMIDLIST pidl = 0;
			ret = SHGetSpecialFolderLocation(0, CSIDL_APPDATA, &pidl);
			if (SUCCEEDED(ret))
			{
				char szPath[_MAX_PATH];
				char szMicrosoftPath[_MAX_PATH];
				char szFrontPagePath[_MAX_PATH];

				*szPath = 0;
        
				if (SHGetPathFromIDList(pidl, szPath))
				{
					strcpy(szMicrosoftPath,szPath);
					strcat(szMicrosoftPath, "\\Microsoft");

					strcpy(szFrontPagePath,szPath);
					strcat(szFrontPagePath, "\\Microsoft\\FrontPage");
				}
        
				// Get the shell's allocator to free PIDLs 
				LPMALLOC lpMalloc; 
				if (!SHGetMalloc(&lpMalloc) && (NULL != lpMalloc))
				{
					if (NULL != pidl)
						lpMalloc->Free(pidl);
					lpMalloc->Release();
				}

				// display relevant paths for FP2K custom components
				if(*szPath)
				{
					printf(pszBots,szRootPath);
					printf("\t(or in the _vti_bot folder inside your web server's root web)\n");

					printf(pszPages,szFrontPagePath);
					printf(pszWebs,szFrontPagePath);
					printf(pszFrames,szFrontPagePath);

					// CSS components are new for FP2K
					printf(pszCSS,szFrontPagePath);

					// Themes are one level up; now shared with other MS apps
					printf(pszThemes,szMicrosoftPath); 
				}
			}
		}
		else
		{
			// display relevant paths for custom components in pre-FP2K

			// no custom bots prior to FP97
			if(!strcmp(szVersion, FP_98) || !strcmp(szVersion, FP_97))
			{
				printf(pszBots,szRootPath);
				printf("\t(or in the _vti_bot folder inside your web server's root web)\n");
			}

			printf(pszPages,szRootPath);
			printf(pszWebs,szRootPath);

			// no Frames components prior to FP98
			if(!strcmp(szVersion, FP_98))
				printf(pszFrames,szRootPath);

			// no CSS components prior to FP2K
			
			// no Themes prior to FP98, when themes were under FPRoot
			if(!strcmp(szVersion, FP_98))
				printf(pszThemes,szRootPath); 
		}
	}
	else
	{
		printf("Unable to determine FrontPageRoot.\n");
	}

    return 0;
}
